//
//  NearestNeighborFilter.m
//  Nearest Neighbor
//
//  Created by Peter Hosey on 2010-02-06.
//  Copyright Peter Hosey 2010. All rights reserved.
//

#import "NearestNeighborFilter.h"
#import <Foundation/Foundation.h>
#import <ApplicationServices/ApplicationServices.h>

@implementation PRHNearestNeighbor

static CIKernel *_NearestNeighborFilterKernel = nil;

- (id)init
{
    if(_NearestNeighborFilterKernel == nil)
    {
		NSBundle    *bundle = [NSBundle bundleForClass:NSClassFromString(@"PRHNearestNeighbor")];
		NSStringEncoding encoding = NSUTF8StringEncoding;
		NSError     *error = nil;
		NSString    *code = [NSString stringWithContentsOfFile:[bundle pathForResource:@"NearestNeighborFilterKernel" ofType:@"cikernel"] encoding:encoding error:&error];
		NSArray     *kernels = [CIKernel kernelsWithString:code];

		_NearestNeighborFilterKernel = [[kernels objectAtIndex:0] retain];
    }
    return [super init];
}

- (NSString *) displayNameForAttributeName:(NSString *)name {
	return NSLocalizedStringFromTableInBundle(name, @"Description", [NSBundle bundleForClass:[self class]], @"Attribute names");
}

- (NSDictionary *)customAttributes {
    return [NSDictionary dictionaryWithObjectsAndKeys:
		[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithDouble:2.0],     kCIAttributeDefault,
			[NSNumber numberWithDouble:1.0],     kCIAttributeIdentity,
			kCIAttributeTypeInteger,             kCIAttributeType,
			[NSNumber numberWithDouble:  0.001], kCIAttributeMin,
			[NSNumber numberWithDouble:  1.000], kCIAttributeSliderMin,
			[NSNumber numberWithDouble:100.000], kCIAttributeSliderMax,
			[self displayNameForAttributeName:@"inputXFactor"], kCIAttributeDisplayName,
			nil],                            @"inputXFactor",

		[NSDictionary dictionaryWithObjectsAndKeys:
			[NSNumber numberWithDouble:2.0],     kCIAttributeDefault,
			[NSNumber numberWithDouble:1.0],     kCIAttributeIdentity,
			kCIAttributeTypeInteger,             kCIAttributeType,
			[NSNumber numberWithDouble:  0.001], kCIAttributeMin,
			[NSNumber numberWithDouble:  1.000], kCIAttributeSliderMin,
			[NSNumber numberWithDouble:100.000], kCIAttributeSliderMax,
			[self displayNameForAttributeName:@"inputYFactor"], kCIAttributeDisplayName,
			nil],                            @"inputYFactor",

		nil];
}

- (CIImage *)outputImage {
    CISampler *src = [CISampler samplerWithImage:inputImage];
	CIFilterShape *definition = [src definition];
	definition = [definition transformBy:CGAffineTransformMakeScale((CGFloat)[inputXFactor doubleValue], (CGFloat)[inputYFactor doubleValue]) interior:NO];

    return [self apply:_NearestNeighborFilterKernel, src,
        inputXFactor,
        inputYFactor,
	    kCIApplyOptionDefinition, definition,
		nil];
}

@end
